<?php

namespace Lumav\EveryPay\Helper;

use Magento\Framework\DataObject;

class ResponseHandler extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var Validator
     */
    protected $validator;

    /**
     * @var ResponseProcessor
     */
    protected $responseProcessor;

    /**
     * @var OrderProcessor
     */
    protected $orderProcessor;

    /**
     * @var \lumav\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param Validator $validator
     * @param ResponseProcessor $responseProcessor
     * @param OrderProcessor $orderProcessor
     * @param \Lumav\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        Validator $validator,
        ResponseProcessor $responseProcessor,
        OrderProcessor $orderProcessor,
        \Lumav\EveryPay\Logger\Logger $logger
    ) {
        $this->validator = $validator;
        $this->responseProcessor = $responseProcessor;
        $this->orderProcessor = $orderProcessor;
        $this->logger = $logger;
        parent::__construct($context);
    }

    /**
     *
     */
    public function handleResponse($response)
    {
        $result = new DataObject();

        $result->setValidationResult($this->validate($response));
        $result->setIsPaymentSuccessful($this->isPaymentSuccessful($response));

        if (!$result->getIsPaymentSuccessful()) {
            $result->setUrl('checkout/cart');
            return $this->cancelOrderAndRestoreQuote($response, $result);
        } else {
            if ($result->getValidationResult() === true) {
                $result->setUrl('checkout/onepage/success');
            } else {
                $this->logger->error('Error while validating response payload', $result->getData());
                $result->setUrl('checkout/onepage/failure');
                $result->setNoticeMsg(__('It seems that something went wrong. Please don\'t hesitate to contact us if any errors occurred during the payment process'));
            }
            return $result;
        }
    }

    /**
     * @param string[string|int| $response
     * @return array|bool
     */
    protected function validate($response)
    {
        try {
            $validationResult = $this->validator->validate($response);

            if ($validationResult === true) {
                $this->responseProcessor->process($response);
            }
        } catch (\Exception $e) {
            $this->logger->error('Exception while processing response', [$e->getMessage(), $e->getTrace()]);
        }
        return $validationResult;
    }

    /**
     * Check whether the payment was successful from the response data
     *
     * @param $response
     * @return bool
     */
    protected function isPaymentSuccessful($response)
    {
        $success = false;

        if (isset($response['payment_state'])) {
            if ($response['payment_state'] == 'settled' || $response['payment_state'] == 'authorized' || $response['payment_state'] == 'sent_for_processing') {
                $success = true;
            } else if ($response['payment_state'] == 'cancelled'
                || $response['payment_state'] == 'waiting_for_3ds_response'
                || $response['payment_state'] == 'failed') {
                $success =  false;
            }
        } else if (isset($response['cancel'])) { //from IFrame
            $success =  false;
        }

        return $success;
    }

    /**
     * Cancel the order, add a notice
     *
     * @param $response
     * @param DataObject $result
     * @return void
     */
    protected function cancelOrderAndRestoreQuote($response, $result)
    {
        try {
            $this->orderProcessor->cancelOrderAndRestoreQuote($response);
            $result->setIsOrderCancelled(true);
            $result->setNoticeMsg(__('Order has been cancelled.'));
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $result->setError($e);
            $result->setErrorMsg($e->getMessage());
            $this->logger->error('Error while cancelling the order', [$response, $e->getMessage(), $e->getTrace()]);
        } catch (\Exception $e) {
            $result->setError($e);
            $result->setErrorMsg(__('Unable to cancel order'));
            $this->logger->error('Error while cancelling the order', [$response, $e->getMessage(), $e->getTrace()]);
        }
        return $result;
    }
}