<?php

namespace Lumav\EveryPay\Model\Form;

use Magento\Framework\Exception\LocalizedException;

class OrderData extends AbstractData
{
    /**
     * @var \Magento\Directory\Api\CountryInformationAcquirerInterface
     */
    protected $country;

    /**
     * @var \Magento\Quote\Model\QuoteRepository
     */
    protected $quoteRepository;

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var \Lumav\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Directory\Api\CountryInformationAcquirerInterface $country
     * @param \Magento\Quote\Model\QuoteRepository $quoteRepository
     * @param \Lumav\EveryPay\Model\Config $config
     * @param \Lumav\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Sales\Model\OrderRepository $orderRepository,
        \Magento\Directory\Api\CountryInformationAcquirerInterface $country,
        \Magento\Quote\Model\QuoteRepository $quoteRepository,
        \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress,
        \Lumav\EveryPay\Model\Config $config,
        \Lumav\EveryPay\Logger\Logger $logger
    ) {
        $this->country = $country;
        $this->remoteAddress = $remoteAddress;
        $this->quoteRepository = $quoteRepository;
        $this->logger = $logger;

        parent::__construct($orderRepository, $config);
    }

    /**
     * @param int $orderId
     * @return array
     */
    public function getFields($orderId)
    {
        $order = $this->orderRepository->get($orderId);
        $requiredFields = $this->getRequiredFields($order);
        $optionalFields = $this->getOptionalFields($order, true);

        return array_merge($requiredFields, $optionalFields);
    }

    /**
     * @param int $orderId
     * @return array
     */
    public function getCitFields($orderId)
    {
        $order = $this->orderRepository->get($orderId);
        $requiredFields = $this->getRequiredFields($order);
        $optionalFields = $this->getOptionalFields($order, false);

        return array_merge($requiredFields, $optionalFields);
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return array
     * @throws LocalizedException
     */
    protected function getRequiredFields(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $requiredFields = [
            'email' => $order->getCustomerEmail(),
            'amount' => $order->getBaseGrandTotal(),
            'order_reference' => $order->getIncrementId(),
            'customer_ip' => $this->remoteAddress->getRemoteAddress(),
            'timestamp' => $this->resolveTimeStamp($order->getEntityId())
        ];

        foreach ($requiredFields as $field => $value) {
            if (empty($value)) {
                throw new LocalizedException(__('Required %1 field is missing', $field));
            }
        }

        return $requiredFields;
    }

    /**
     * @param int $orderId
     * @return false|int
     */
    protected function resolveTimeStamp($orderId)
    {
        return date('Y-m-d\TH:i:s.P', strtotime($this->orderRepository->get($orderId)->getCreatedAt()));
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @param bool $includeShipping
     * @return array
     */
    protected function getOptionalFields(\Magento\Sales\Api\Data\OrderInterface $order, $includeShipping)
    {
        return $includeShipping 
            ? array_merge($this->getBillingAddressFields($order), $this->getShippingAddressFields($order))
            : $this->getBillingAddressFields($order);
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return array
     */
    protected function getBillingAddressFields(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $fields = [
            'billing_city' => $order->getBillingAddress()->getCity(),
            'billing_country' => $this->country
                ->getCountryInfo($order->getBillingAddress()->getCountryId())->getTwoLetterAbbreviation(),
            'billing_postcode' => $order->getBillingAddress()->getPostcode(),
            'billing_state' => $order->getBillingAddress()->getRegion()
        ];

        $fields = array_merge($fields, $this->getStreetFields($order->getBillingAddress(), 'billing'));

        return $fields;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return array
     */
    protected function getShippingAddressFields(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $shippingAddress = null;

        try {
            $shippingAddress = $this->quoteRepository->get($order->getQuoteId())->getShippingAddress();
        } catch (\Exception $e) {
            $this->logger
                 ->error("Error while retrieving quote's shipping address", [$e->getMessage(), $e->getTrace()]);
        }

        $fields = [];

        if ($shippingAddress && !$shippingAddress->isEmpty()) {
            try {
                $fields = [
                    'shipping_city' => $shippingAddress->getCity(),
                    'shipping_country' => $this->country
                        ->getCountryInfo($shippingAddress->getCountryId())->getTwoLetterAbbreviation(),
                    'shipping_postcode' => $shippingAddress->getPostcode(),
                    'shipping_state' => $shippingAddress->getRegion()
                ];

                $fields = array_merge($fields, $this->getStreetFields($shippingAddress, 'shipping'));
            } catch (\Exception $e) {
                $this->logger
                     ->error("Error while retrieving quote's shipping address", [$e->getMessage(), $e->getTrace()]);
            }
        }

        return $fields;
    }

    /**
     * @param \Magento\Sales\Model\Order\Address $address
     * @param string $type
     * @return array
     */
    protected function getStreetFields($address, $type)
    {
        $data = [];
        $streetLines = $address->getStreet();
        for ($i = 0; $i < 3; $i++) {
            $data[$type . '_line' . ($i + 1)] = isset($streetLines[$i]) ? $streetLines[$i] : '';
        }
        return $data;
    }

    /**
     * @param int $orderId
     * @return array
     */
    public function getAppleOrderData($orderId)
    {
        $order = $this->orderRepository->get($orderId);
        return [
            'country_code' => $order->getBillingAddress()->getCountryId(),
            'currency_code' => $order->getOrderCurrencyCode(),
            'total_amount' => $order->getBaseGrandTotal(),
            'total_label' => $order->getBaseGrandTotal() . ' ' . $order->getOrderCurrencyCode()
        ];
    }
}
