<?php

namespace Lumav\EveryPay\Model;

use Lumav\EveryPay\Model\Config;
use Magento\Framework\HTTP\ClientFactory;
use Magento\Framework\Exception\LocalizedException;
use Lumav\EveryPay\Model\Form\FormDataBuilder;
use Lumav\EveryPay\Helper\ResponseHandler;
use Lumav\EveryPay\Logger\Logger;

class Gateway
{
	const CARDS = 'cards';
	const BANKS = 'banks';
	const ALTERNATIVE = 'alternative';
	const APPLE = 'apple';
	const GOOGLE = 'google';

	const PROCESSING_ACCOUNTS = 'processing_accounts/';
	const ONEOFF = 'payments/oneoff/';
	const CIT = 'payments/cit/';
	const PAYMENT_REFERENCE = 'payments/';

	const PAYMENT_SUCCESS = 'success';
	const PAYMENT_FAILURE = 'fail';

	/**
	 * @var Config
	 */
	protected $config;

	/**
	 * @var ClientFactory
	 */
	protected $curlFactory;

	/**
	 * @var FormDataBuilder
	 */
	protected $dataBuilder;

	/**
	 * @var ResponseHandler
	 */
	protected $responseHandler;

	/**
	 * @var Logger
	 */
	protected $logger;

	/**
	 * @param Config $config
	 * @param ClientFactory $curlFactory
	 * @param FormDataBuilder $dataBuilder
	 * @param ResponseHandler $responseHandler
	 * @param Logger $logger
	 */
	public function __construct(
		Config $config,
		ClientFactory $curlFactory,
		FormDataBuilder $dataBuilder,
		ResponseHandler $responseHandler,
		Logger $logger
	) {
		$this->config = $config;
		$this->curlFactory = $curlFactory;
		$this->dataBuilder = $dataBuilder;
		$this->responseHandler = $responseHandler;
		$this->logger = $logger;
	}

	/**
	 * Request payment status & update order
	 *
	 * @param string $reference
	 * @return bool
	 * @throws LocalizedException
	 */
	public function updatePaymentStatus($reference)
	{
		if ($reference) {
			$endpoint = self::PAYMENT_REFERENCE . $reference;
			$this->logger->info(__('Updating payment status, payment_reference: ' . $reference));
			$response = $this->apiCall($endpoint, 'GET', $this->dataBuilder->getPaymentStatusFields($reference));
			$responseData = json_decode($response, true);
			$this->logger->info(sprintf(__('Update payment status request response with payment_reference %s: '), $reference));
			$this->logger->info(__(json_encode($responseData, JSON_PRETTY_PRINT)));

			$result = $this->responseHandler->handleResponse($responseData);

			if ($result->getIsPaymentSuccessful()) {
	            if (is_array($result->getValidationResult())) {
	                foreach ($result->getValidationResult() as $error) {
	                	$this->logger->error(__($error));
	                }
	                return false;
	            }
	            return true;
	        } else {
	        	if ($result->getIsOrderCancelled()) {
	        		$this->logger->info($result->getNoticeMsg());
	        	} else {
	        		$this->logger->error($result->getErrorMsg());
	        	}
	        	return true;
	        }
		} else {
			$errMsg = __('No payment reference specified.');
			$this->logger->info($errMsg);
			throw new LocalizedException($errMsg);
		}
	}

	/**
	 * Initialize payment
	 *
	 * @param int $orderId
	 * @param string $methodId
	 * @param null|int $tokenId
	 * @return string $redirectUrl
	 */
	public function initPayment($orderId, $methodId, $tokenId = null)
	{
		if ($tokenId && $tokenId !== Config::NEW_CARD_ID && $methodId === 'card') {
			$response = $this->makeCitPayment($orderId, $methodId, $tokenId);
			return is_array($response) ? json_encode($response) : $response;
		} else {
			return $this->makeOneoffPayment($orderId, $methodId);
		}
	}

	/**
	 * Make oneoff payment
	 *
	 * @param int $orderId
	 * @param string $methodId
	 * @return string
	 * @throws LocalizedException
	 */
	protected function makeOneoffPayment($orderId, $methodId)
	{
		$this->logger->info(sprintf(__('Making oneoff payment request for order: %s, payment method: %s'), $orderId, $methodId));
		$data = $this->dataBuilder->getOneoffFields($orderId);
		if($methodId == self::APPLE || $methodId == self::GOOGLE) {
		    $data['mobile_payment'] = true;
		    unset($data['request_token']);
		}
		$this->logger->info(sprintf(__('Request body: %s'), json_encode($data, JSON_PRETTY_PRINT)));
		$response = $this->apiCall(self::ONEOFF, 'POST', $data);
		$responseData = json_decode($response, true);
		$this->logger->info(sprintf(__('Oneoff request response for order %s:'), $orderId));
		$this->logger->info(__(json_encode($responseData, JSON_PRETTY_PRINT)));
		if (!isset($responseData['payment_methods'])) {
			$this->logger->error(sprintf(__('Missing payment_methods field from oneoff request response, order %s'), $orderId));
			throw new LocalizedException(__('Payment initialization failed!'));
		}
		if($methodId == self::APPLE) {
		    return $this->getAppleReturnData($responseData, $orderId);
		}
		if($methodId == self::GOOGLE) {
		    return $this->getGoogleReturnData($responseData, $orderId);
		}
		foreach ($responseData['payment_methods'] as $method) {
			if ($method['source'] === $methodId) {
				return $method['payment_link'];
			}
		}
	}

	/**
	 * Make CIT payment
	 *
	 * @param int $orderId
	 * @param string $methodId
	 * @param int $tokenId
	 * @return string|array
	 * @throws LocalizedException
	 */
	protected function makeCitPayment($orderId, $methodId, $tokenId)
	{
		$this->logger->info(sprintf(__('Making CIT payment request for order: %s'), $orderId));
		$data = $this->dataBuilder->getCitFields($orderId, $tokenId);
		$this->logger->info(sprintf(__('Request body: %s'), json_encode($data, JSON_PRETTY_PRINT)));
		$response = $this->apiCall(self::CIT, 'POST', $data);
		$responseData = json_decode($response, true);
		$this->logger->info(sprintf(__('CIT request response for order %s:'), $orderId));
		$this->logger->info(__(json_encode($responseData, JSON_PRETTY_PRINT)));
		if (isset($responseData['payment_link'])) {
			return $responseData['payment_link'];
		} elseif (isset($responseData['payment_method'])) {
			//handle completed payment response
			$result = $this->responseHandler->handleResponse($responseData);

			if ($result->getIsPaymentSuccessful()) {
	            if (is_array($result->getValidationResult())) {
	                return [
	                	'errors' => $result->getValidationResult(),
	                	'msg' => $result->getNoticeMsg(),
	                	'url' => $result->getUrl()
	                ];
	            }
	            return $result->getUrl();
	        }
	        //payment was not successful
	        return [
	        	'errors' => [],
	        	'msg' => $result->getIsOrderCancelled() ? $result->getNoticeMsg() : $result->getErrorMsg(),
	        	'url' => $result->getUrl()
	        ];
		} else {
			throw new LocalizedException(__('Payment initialization failed!'));
		}
	}

	/**
	 * Update payment method configuration
	 *
	 * @return $this
	 */
	public function updatePaymentMethods()
	{
		$accountData = $this->getAccountData();

		$supportedCountries = [];
		$paymentMethods = [self::CARDS => [], self::BANKS => [], self::ALTERNATIVE => []];
		
		foreach ($accountData['payment_methods'] as $data) {
			$source = isset($data['source']) ? $data['source'] : null;
			if ($source === 'card') {
				$paymentMethods[self::CARDS][] = $data;
			} elseif (strpos($source, '_ob_') !== false && strpos($source, '_ob_') !== 0) {
				$paymentMethods[self::BANKS][] = $data;
			} else {
				$paymentMethods[self::ALTERNATIVE][] = $data;
			}

			if (!in_array($data['country_code'], $supportedCountries)) {
				$supportedCountries[] = $data['country_code'];
			}
		}
		
		$paymentMethods = json_encode($paymentMethods);
		$this->logger->info($paymentMethods);
		$this->config->setPaymentMethods($paymentMethods);
		$this->config->setSupportedCountries($supportedCountries);
		return $this;
	}
	
	/**
	 * Return ApplePay parameters from oneoff payment data
	 *
	 * @param array $responseData
	 * @param int $orderId
	 * @return bool|array
	 */
	public function getAppleReturnData($responseData, $orderId)
	{
	    $merchantId = $this->config->getApplePayMerchantId();
	    if($merchantId && !empty($responseData['account_name']) && !empty($responseData['payment_reference']) && !empty($responseData['payment_link']) && !empty($responseData['api_username']) && !empty($responseData['mobile_access_token'])) {
	        $orderData = $this->dataBuilder->getAppleOrderData($orderId);
	        $applePayData = [
                'merchant_id' => $merchantId,
                'account_name' => $responseData['account_name'],
                'payment_reference' => $responseData['payment_reference'],
                'payment_link' => $responseData['payment_link'],
                'api_username' => $responseData['api_username'],
                'access_token' => $responseData['mobile_access_token'],
                'country_code' => $responseData['descriptor_country'] ?? $orderData['country_code'],
                'currency_code' => $responseData['currency'] ?? $orderData['currency_code'],
            ];
	        return [array_merge($orderData,$applePayData)];
	    }
	    
	    return false;
	}
	
	/**
	 * Return Google Pay parameters from oneoff payment data
	 *
	 * @param array $responseData
	 * @param int $orderId
	 * @return bool|array
	 */
	public function getGoogleReturnData($responseData, $orderId)
	{
	    if(!empty($responseData['account_name']) && !empty($responseData['payment_reference']) && !empty($responseData['payment_link']) && !empty($responseData['api_username']) && !empty($responseData['mobile_access_token'])) {
	        $orderData = $this->dataBuilder->getAppleOrderData($orderId);
	        $googlePayData = [
                'account_name' => $responseData['account_name'],
                'payment_reference' => $responseData['payment_reference'],
                'payment_link' => $responseData['payment_link'],
                'api_username' => $responseData['api_username'],
                'access_token' => $responseData['mobile_access_token'],
                'country_code' => $responseData['descriptor_country'] ?? $orderData['country_code'],
                'currency_code' => $responseData['currency'] ?? $orderData['currency_code'],
            ];
	        return [array_merge($orderData,$googlePayData)];
	    }
	    
	    return false;
	}

	/**
	 * Get processing account data from API
	 *
	 * @return array
	 * @throws LocalizedException
	 */
	protected function getAccountData()
	{
		if ($accountName = $this->config->getProcessingAccount()) {
			$endpoint = self::PROCESSING_ACCOUNTS . $accountName;
			$this->logger->info(__('Updating Everypay payment methods for account ' . $accountName));
			$result = $this->apiCall($endpoint, 'GET', $this->dataBuilder->getProcessingAccountFields());
			return json_decode($result, true);
		}
		$errMsg = __('Processing account name is not configured.');
		$this->logger->info($errMsg);
		throw new LocalizedException($errMsg);
	}

	/**
	 * Send curl request to json REST API
	 *
	 * @param string $endpoint
	 * @param string $method
	 * @param array $data
	 * @return string
	 * @throws LocalizedException
	 */
	protected function apiCall($endpoint, $method, $data = null)
	{
		$url = $endpoint == self::ONEOFF ? $this->config->getOneOffApiEndpoint() : $this->config->getApiEndpoint();
		$user = $this->config->getApiUser();
		$pswd = $this->config->getApiPassword();

		if ($url && $user && $pswd) {
			$url .= $endpoint;
			$curl = $this->curlFactory->create();
			$curl->setHeaders(['Content-Type: application/json', 'Accept: application/json']);
			$curl->setCredentials($user, $pswd);

		    switch ($method) {
		        case "POST":
		            $curl->post($url, $data);
		            break;
		        default:
		            if ($data) {
		                $url = sprintf("%s?%s", $url, http_build_query($data));
		            }
		            $curl->get($url);
		    }
		    return $curl->getBody();
		}
		$errMsg = __('Call to Everypay API failed. Gateway URL or credentials are not configured.');
		$this->logger->info($errMsg);
		throw new LocalizedException($errMsg);
	}
}